"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path_1 = require("path");
function createCache(cacheRoot) {
    function getFilename(key) {
        return path_1.join(cacheRoot, key);
    }
    function save(content, key, timestamp) {
        return __awaiter(this, void 0, void 0, function* () {
            const filePath = getFilename(key);
            yield fs.mkdirp(path_1.dirname(filePath));
            const withTime = { timestamp: timestamp.toUTCString(), content };
            yield fs.writeFile(filePath, JSON.stringify(withTime), { encoding: "utf8" });
        });
    }
    function load(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const path = getFilename(key);
            if (yield fs.pathExists(path)) {
                const result = JSON.parse(yield fs.readFile(path, "utf8"));
                result.exists = true;
                return result;
            }
            else {
                return {
                    exists: false,
                };
            }
        });
    }
    return {
        save,
        load,
    };
}
exports.createCache = createCache;
//# sourceMappingURL=cache.js.map