"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const https = require("https");
const build_path_1 = require("./build-path");
const github_1 = require("./github");
let oauthToken;
function initialize(_oauthToken) {
    oauthToken = _oauthToken;
}
exports.initialize = initialize;
let me;
function getMyLogin() {
    return __awaiter(this, void 0, void 0, function* () {
        if (me === undefined) {
            me = (yield parseGet("/user")).login;
        }
        return me;
    });
}
exports.getMyLogin = getMyLogin;
function addLabels(issue, labels) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/labels/#add-labels-to-an-issue
        const body = JSON.stringify(labels);
        yield exec("POST", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "labels"), { body });
    });
}
exports.addLabels = addLabels;
function removeLabels(issue, labels) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/labels/#remove-a-label-from-an-issue
        for (const label of labels) {
            yield exec("DELETE", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "labels", label));
        }
    });
}
exports.removeLabels = removeLabels;
function setLabels(issue, labels) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/labels/#replace-all-labels-for-an-issue
        const body = JSON.stringify(labels);
        yield exec("PUT", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "labels"), { body });
    });
}
exports.setLabels = setLabels;
function lockIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec("PUT", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "lock"), { body: "" });
    });
}
exports.lockIssue = lockIssue;
function unlockIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec("DELETE", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "lock"));
    });
}
exports.unlockIssue = unlockIssue;
function closeIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        const body = JSON.stringify({ state: "closed" });
        yield exec("PATCH", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number), { body });
    });
}
exports.closeIssue = closeIssue;
function reopenIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        const body = JSON.stringify({ state: "open" });
        yield exec("PATCH", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number), { body });
    });
}
exports.reopenIssue = reopenIssue;
function addComment(issue, body) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec("POST", build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "comments"), { body: JSON.stringify({ body }) });
    });
}
exports.addComment = addComment;
function editComment(comment, body) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec("PATCH", build_path_1.default("repos", comment.repository.owner, comment.repository.name, "issues", "comments", comment.id), { body: JSON.stringify({ body }) });
    });
}
exports.editComment = editComment;
function fetchAllIssuesAndPRsRaw(repo) {
    return __awaiter(this, void 0, void 0, function* () {
        return fetchPage(1);
        function fetchPage(page) {
            return __awaiter(this, void 0, void 0, function* () {
                // https://developer.github.com/v3/issues/#list-issues
                const queryString = {
                    sort: "created",
                    filter: "all",
                    direction: "asc",
                    per_page: 100,
                    page,
                };
                const thisPage = yield parseGet(build_path_1.default("repos", repo.owner, repo.name, "issues"), { queryString });
                return {
                    page: thisPage,
                    next: thisPage.length === 100 ? (() => fetchPage(page + 1)) : undefined,
                };
            });
        }
    });
}
exports.fetchAllIssuesAndPRsRaw = fetchAllIssuesAndPRsRaw;
function fetchChangedIssuesRaw(repo) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/#list-issues
        const queryString = {
            sort: "updated",
            filter: "all",
            direction: "desc",
            per_page: 100,
        };
        const page = yield parseGet(build_path_1.default("repos", repo.owner, repo.name, "issues"), { queryString });
        return page.filter((i) => !i.pull_request);
    });
}
exports.fetchChangedIssuesRaw = fetchChangedIssuesRaw;
function runQuery(q, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        if (q.kind === "prs") {
            // https://developer.github.com/v3/pulls/#list-pull-requests
            let count = 0;
            let pageNumber = 1;
            const repo = github_1.parseBasicRepoReference(q.repo);
            while (true) {
                const queryString = {
                    sort: q.sort,
                    state: q.state,
                    direction: q.direction,
                    per_page: 100,
                    page: pageNumber,
                };
                const ref = github_1.parseBasicRepoReference(q.repo);
                const page = yield parseGet(build_path_1.default("repos", ref.owner, ref.name, "pulls"), { queryString });
                for (const item of page) {
                    const pr = yield github_1.PullRequest.fromReference(repo, item.number);
                    yield callback(pr);
                    count++;
                    if (count === q.count)
                        return;
                }
                // Exhausted the query
                if (page.length < 100)
                    return;
                pageNumber++;
            }
        }
        else {
            throw new Error("Other query kinds NYI");
        }
    });
}
exports.runQuery = runQuery;
function fetchChangedPRsRaw(repo) {
    // https://developer.github.com/v3/pulls/#list-pull-requests
    const queryString = {
        sort: "updated",
        state: "all",
        direction: "desc",
        per_page: 100,
    };
    return parseGet(build_path_1.default("repos", repo.owner, repo.name, "pulls"), { queryString });
}
exports.fetchChangedPRsRaw = fetchChangedPRsRaw;
const ProjectsPreview = "application/vnd.github.inertia-preview+json";
function fetchProjectColumns(projectId) {
    return parseGet(build_path_1.default("projects", projectId, "columns"), { preview: ProjectsPreview });
}
exports.fetchProjectColumns = fetchProjectColumns;
function fetchProjectColumnCards(columnId) {
    console.log(`Fetch column cards for column ${columnId}`);
    return parseGetPaged(build_path_1.default("projects", "columns", columnId, "cards"), 30, { preview: ProjectsPreview });
}
exports.fetchProjectColumnCards = fetchProjectColumnCards;
function createProjectCard(columnId, issue) {
    return __awaiter(this, void 0, void 0, function* () {
        const body = JSON.stringify({ content_id: issue.id, content_type: "Issue" });
        yield exec("POST", build_path_1.default("projects", "columns", columnId, "cards"), {
            preview: ProjectsPreview,
            body,
        });
    });
}
exports.createProjectCard = createProjectCard;
function moveProjectCard(card, targetColumn) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec("POST", build_path_1.default("projects", "columns", "cards", card.id, "moves"), {
            preview: ProjectsPreview,
            body: JSON.stringify({
                position: "bottom",
                column_id: targetColumn.columnId,
            }),
        });
    });
}
exports.moveProjectCard = moveProjectCard;
function deleteProjectCard(card) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec("DELETE", build_path_1.default("projects", "columns", "cards", card.id), {
            preview: ProjectsPreview,
        });
    });
}
exports.deleteProjectCard = deleteProjectCard;
function fetchIssueComments(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        const raw = yield execPaged(build_path_1.default("repos", issue.repository.owner, issue.repository.name, "issues", issue.number, "comments"));
        return raw;
    });
}
exports.fetchIssueComments = fetchIssueComments;
function fetchPRCommits(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/pulls/#list-commits-on-a-pull-request
        const raw = yield execPaged(build_path_1.default("repos", issue.repository.owner, issue.repository.name, "pulls", issue.number, "commits"));
        return raw;
    });
}
exports.fetchPRCommits = fetchPRCommits;
function fetchPRFiles(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        const raw = yield execPaged(build_path_1.default("repos", issue.repository.owner, issue.repository.name, "pulls", issue.number, "files"));
        return raw;
    });
}
exports.fetchPRFiles = fetchPRFiles;
function fetchIssueCommentReactions(repo, commentId) {
    // https://developer.github.com/v3/reactions/#list-reactions-for-an-issue-comment
    // GET /repos/:owner/:repo/issues/comments/:id/reactions
    return parseGet(build_path_1.default("repos", repo.owner, repo.name, "issues", "comments", commentId, "reactions"));
}
exports.fetchIssueCommentReactions = fetchIssueCommentReactions;
function fetchPR(repo, number) {
    return __awaiter(this, void 0, void 0, function* () {
        return parseGet(build_path_1.default("repos", repo.owner, repo.name, "pulls", number));
    });
}
exports.fetchPR = fetchPR;
function mergePR(pr, { title, message, sha }) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/pulls/#merge-a-pull-request-merge-button
        yield exec("PUT", build_path_1.default("repos", pr.repository.owner, pr.repository.name, "pulls", pr.number, "merge"), {
            body: JSON.stringify({
                commit_title: title,
                commit_message: message,
                sha,
                merge_method: "squash",
            }),
        });
    });
}
exports.mergePR = mergePR;
function fetchIssue(repo, number) {
    return parseGet(build_path_1.default("repos", repo.owner, repo.name, "issues", number));
}
exports.fetchIssue = fetchIssue;
function fetchPRReviews(repo, number) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = build_path_1.default("repos", repo.owner, repo.name, "pulls", number, "reviews");
        // https://developer.github.com/v3/pulls/reviews/#list-reviews-on-a-pull-request
        // GET /repos/:owner/:repo/pulls/:number/reviews
        try {
            return yield parseGet(url, {
                preview: "application/vnd.github.black-cat-preview+json",
            });
        }
        catch (e) {
            if (e instanceof ExecError && e.statusCode === 404) {
                // No reviews
                return [];
            }
            throw e;
        }
    });
}
exports.fetchPRReviews = fetchPRReviews;
function fetchRefStatusSummary(repo, ref) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/repos/statuses/#get-the-combined-status-for-a-specific-ref
        // GET /repos/:owner/:repo/commits/:ref/status
        return parseGet(build_path_1.default("repos", repo.owner, repo.name, "commits", ref, "status"));
    });
}
exports.fetchRefStatusSummary = fetchRefStatusSummary;
function execPaged(path, perPage = 100, opts = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = [];
        let pageNumber = 1;
        while (true) {
            console.log(`Fetch page ${pageNumber}...`);
            const qs = { page: pageNumber.toString(), per_page: perPage.toString() };
            const arr = yield parseGet(path, Object.assign({}, opts, { queryString: qs }));
            if (!Array.isArray(arr)) {
                throw new Error("Didn't parse an array from a paged fetch");
            }
            result.push(...arr);
            if (arr.length < perPage) {
                return result;
            }
            pageNumber++;
        }
    });
}
class ExecError extends Error {
    constructor(message, statusCode) {
        super(message);
        this.statusCode = statusCode;
    }
}
exports.ExecError = ExecError;
let lastRateLimit = -1; // starts off unknown
let lastRateLimitRemaining = -1;
function exec(method, basePath, opts = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const hostname = "api.github.com";
        const bodyStream = opts.body === undefined ? undefined : Buffer.from(opts.body);
        const path = getPathWithQuery(basePath, opts.queryString);
        console.log(`[${lastRateLimitRemaining} / ${lastRateLimit}] HTTPS: ${method} https://${hostname}${path}`);
        if (opts.body) {
            console.log(` POST -> ${opts.body}`);
        }
        const headers = getHeaders(bodyStream, opts.preview);
        return new Promise((resolve, reject) => {
            const req = https.request({ method, path, headers, hostname }, (res) => {
                // console.log('Headers: ' + JSON.stringify(res.headers, undefined, 2));
                lastRateLimit = parseIntFromHeader(res.headers["x-ratelimit-limit"]);
                lastRateLimitRemaining = parseIntFromHeader(res.headers["x-ratelimit-remaining"]);
                const { statusCode } = res;
                if (statusCode === undefined) {
                    throw new Error();
                }
                if (statusCode >= 400) {
                    reject(new ExecError(`Status code ${statusCode} returned from ${basePath}`, statusCode));
                    return;
                }
                res.setEncoding("utf8");
                let data = "";
                res.on("data", (chunk) => {
                    data = data + chunk;
                });
                res.on("end", () => {
                    resolve(data);
                });
                res.on("error", (err) => {
                    console.error("Connection Error!");
                    console.error(err);
                    reject(err);
                });
            });
            req.on("error", (e) => {
                console.error(e);
                reject(e);
            });
            if (bodyStream !== undefined) {
                req.write(bodyStream);
            }
            req.end();
        });
    });
}
exports.exec = exec;
function parseIntFromHeader(header) {
    if (typeof header !== "string")
        throw new Error();
    const int = Number.parseInt(header);
    if (Number.isNaN(int))
        throw new Error();
    return int;
}
function getPathWithQuery(basePath, queryString) {
    let fullPath = basePath;
    if (queryString && Object.keys(queryString).length > 0) {
        const encoded = Object.keys(queryString).map((k) => k + "=" + encodeURIComponent(queryString[k])).join("&");
        fullPath = fullPath + "?" + encoded;
    }
    return fullPath;
}
function getHeaders(bodyStream, preview) {
    const headers = {
        "User-Agent": "RyanCavanaugh idembot",
        "Accept": preview || "application/vnd.github.squirrel-girl-preview+json",
        "Authorization": `token ${oauthToken}`,
    };
    if (bodyStream !== undefined) {
        headers["Content-Type"] = "application/json";
        headers["Content-Length"] = bodyStream.length;
    }
    return headers;
}
function parseGet(path, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        return JSON.parse(yield exec("GET", path, opts));
    });
}
function parseGetPaged(path, pageSize, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield execPaged(path, pageSize, opts);
    });
}
//# sourceMappingURL=client.js.map